             .title        "MMC2 Reader Plugin"


;12/01/05 
;Written by BootGod

;01/01/06
;Bugfix by BootGod
;Fixed a bug that could cause first bank to be corrupt
             
;vectors for CopyNES BIOS calls

send_byte:   .equ 0200h		;sends byte to client
baton:       .equ 0203h		;indicates xfer progress on LCD if installed
wr_ppu:      .equ 020ch		;sets address for VRAM access

;variables

read_addr:   .equ 00e0h
read_addr_lo:.equ 00e0h
read_addr_hi:.equ 00e1h
banks_left:  .equ 00e2h
cur_bank:    .equ 00e3h
romsize:     .equ 00e4h

;plugin header defines
p_ver: = 2	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Nintendo PxROM / MMC2 carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 128 >> 4,0 >> 4
             .db 128 >> 3,0 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here

;mirroring detection
;Horizontal	(x = 1)
;Vertical	(x = 0)
;Four-Screen	(x = 2)
;MapperCtrl	(x = 4)

;really this mirroring check is unnecessary, as the MMC2 controls it so you could just as well do
;lda #4
;jsr send_byte
;but this I left it in anyways as its a good reference

             ldx #02fh
init_md:     txa		;load first byte of 2000, 2400, 2800, 2c00 w/ 00h
             jsr wr_ppu
             lda #0
             sta 2007h
             dex
             cpx #01fh
             bne init_md

             ldx #0		;start off with check for H mirroring

             lda #020h
             jsr wr_ppu
             lda #055h
             sta 2007h		;write a value to $2000

             lda #028h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #055h
             beq chk_mmc_mirr	;if value appears at $2800, then we have H mirroring

             inx		;next check for V mirroring]

             lda #024h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #055h
             beq chk_mmc_mirr	;if value appears at $2400, then we have V mirroring

             inx		;it wasn't mirrorred so it's 4 screen
             bne send_mirr
             
chk_mmc_mirr:lda #01h		; check if can be controlled by MMC ($f000 is MMC2 mirr reg)
             sta 0f000h
             lda #024h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #055h
             bne send_mirr

             lda #00h
             sta 0f000h
             lda #028h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #055h
             bne send_mirr	;test for MMC mirror control

             ldx #04h

send_mirr:   txa
             jsr send_byte

;Set up for PRG transfer

prg_info:    lda #0
             jsr send_byte	;send romsize_lo (0)
             lda #2
             sta romsize
             jsr send_byte	;send romsize_hi

             lda #16		;# of 8K banks (16 * 8K = 128K)
             sta banks_left
             lda #0
             sta cur_bank

             lda #1		;send type is PRG
             jsr send_byte

set_prg_bank:lda cur_bank
             sta 0a000h		;write bank num to MMC2 PRG select
             inc cur_bank
             lda #0		;set read address to $8000
             sta read_addr_lo
             lda #080h
             sta read_addr_hi

             ldx #32		;set counters for 8K bank xfer
             ldy #0

send_prg:    lda (read_addr),y
             jsr send_byte
             iny
             bne send_prg
             inc read_addr_hi
             jsr baton
             dex
             bne send_prg
             dec banks_left
             bne set_prg_bank	;8K has been sent, load next bank
             
;Set up for CHR transfer
                         
chr_info:    lda #0
             jsr send_byte	;send romsize_lo (0)
             lda #2
             sta romsize
             jsr send_byte	;send romsize_hi

             lda #32		;# of 4K banks (32 * 4K = 128K)
             sta banks_left
             lda #0
             sta cur_bank

             lda #2		;send type is CHR
             jsr send_byte

set_chr_bank:lda cur_bank
             sta 0b000h
             sta 0c000h		;write bank num to MMC2 CHR select

             lda #00h		;init ppu read
             jsr wr_ppu
             lda 2007h

             ldx #16		;set counters for 4K bank xfer
             ldy #0

send_chr:    lda 2007h
             jsr send_byte
             iny
             bne send_chr
             jsr baton
             dex
             bne send_chr
             inc cur_bank
             dec banks_left
             bne set_chr_bank	;4K has been sent, load next bank

;Signal client no more is coming

done:        lda #0
             jsr send_byte
             lda #0
             jsr send_byte
             lda #0
             jsr send_byte

             rts


             .fill 0800h-*,0ffh	;fill rest to get 1K of data

             .end
